import logging

import tensorflow as tf

from transformers_keras.datasets.abstract_dataset_builder import AbstractDatasetBuilder
from transformers_keras.datasets.bert_dataset_builder import BertTFRecordDatasetBuilder
from transformers_keras.datasets.transformer_dataset_builder import (
    TransformerDatasetBuilder,
    TransformerTextFileDatasetBuilder,
    TransformerTFRecordDatasetBuilder,
)
from transformers_keras.tokenizers.bert_tokenizer import BertAbstractTokenizer, BertDefaultTokenizer, BertVocabBasedTokenizer
from transformers_keras.tokenizers.tokenizer import BasicTokenizer, WordpieceTokenizer
from transformers_keras.tokenizers.transformer_tokenizer import (
    TransformerAbstractTokenizer,
    TransformerDefaultTokenizer,
    TransformerJiebaTokenizer,
    TransformerVocabBasedTokenizer,
)

from .callbacks import SavedModelExporter, TransformerLearningRate
from .layers import DecoderLayer, EncoderLayer, MultiHeadAttention, PointWiseFeedForwardNetwork, ScaledDotProductAttention
from .losses import MaskedSparseCategoricalCrossentropy
from .metrics import MaskedSparseCategoricalAccuracy
from .modeling_albert import (
    Albert4PreTraining,
    AlbertEmbedding,
    AlbertEncoder,
    AlbertEncoderGroup,
    AlbertEncoderLayer,
    AlbertMLMHead,
    AlbertModel,
    AlbertSOPHead,
)
from .modeling_bert import Bert, Bert4PreTraining, BertEmbedding, BertMLMHead, BertNSPHead
from .modeling_transformer import PositionalEncoding, Transformer, TransformerDecoder, TransformerEmbedding, TransformerEncoder
from .runners import AlbertRunner, BertRunner, TransformerRunner

__name__ = 'transformers_keras'
r_version__ = '0.1.3'

logging.basicConfig(format="%(asctime)s %(levelname)s %(filename)15s %(lineno)4d] %(message)s", level=logging.INFO)


def build_pretraining_bert_model(model_config):
    max_sequence_length = model_config.get('max_positions', 512)
    input_ids = tf.keras.layers.Input(
        shape=(max_sequence_length,), dtype=tf.int32, name='input_ids')
    input_mask = tf.keras.layers.Input(
        shape=(max_sequence_length,), dtype=tf.int32, name='input_mask')
    segment_ids = tf.keras.layers.Input(
        shape=(max_sequence_length,), dtype=tf.int32, name='segment_ids')

    inputs = (input_ids, segment_ids, input_mask)
    bert = Bert4PreTraining(**model_config)
    outputs = bert(inputs)

    predictions = tf.keras.layers.Lambda(lambda x: x, name='predictions')(outputs[0])
    relations = tf.keras.layers.Lambda(lambda x: x, name='relations')(outputs[1])

    model = tf.keras.Model(inputs=inputs, outputs=[predictions, relations])
    lr = model_config.get('learning_rate', 3e-5)
    epsilon = model_config.get('epsilon', 1e-12)
    clipnorm = model_config.get('clipnorm', 1.0)
    model.compile(
        optimizer=tf.keras.optimizers.Adam(learning_rate=lr, epsilon=epsilon, clipnorm=clipnorm),
        loss={
            'predictions': MaskedSparseCategoricalCrossentropy(mask_id=0, from_logits=True, name='pred_loss'),
            'relations': tf.keras.losses.CategoricalCrossentropy(from_logits=True, name='rel_loss'),
        },
        metrics={
            'predictions': [
                MaskedSparseCategoricalAccuracy(mask_id=0, from_logits=True, name='pred_acc'),
            ],
            'relations': [
                tf.keras.metrics.CategoricalAccuracy(name='rel_acc'),
            ]
        })
    model.summary()
    return model


def build_pretraining_albert_model(model_config):
    max_sequence_length = model_config.get('max_positions', 512)
    input_ids = tf.keras.layers.Input(
        shape=(max_sequence_length,), dtype=tf.int32, name='input_ids')
    input_mask = tf.keras.layers.Input(
        shape=(max_sequence_length,), dtype=tf.int32, name='input_mask')
    segment_ids = tf.keras.layers.Input(
        shape=(max_sequence_length,), dtype=tf.int32, name='segment_ids')

    inputs = (input_ids, segment_ids, input_mask)
    albert = Albert4PreTraining(**model_config)
    predictions, relations, all_states, all_attn_weights = albert(inputs=inputs)

    predictions = tf.keras.layers.Lambda(lambda x: x, name='predictions')(predictions)
    relations = tf.keras.layers.Lambda(lambda x: x, name='relations')(relations)

    model = tf.keras.Model(
        inputs=[input_ids, segment_ids, input_mask], outputs=[predictions, relations])

    lr = model_config.get('learning_rate', 3e-5)
    epsilon = model_config.get('epsilon', 1e-12)
    clipnorm = model_config.get('clipnorm', 1.0)

    model.compile(
        optimizer=tf.keras.optimizers.Adam(learning_rate=lr, epsilon=epsilon, clipnorm=clipnorm),
        loss={
            'predictions': MaskedSparseCategoricalCrossentropy(
                mask_id=0, from_logits=True, name='pred_loss'),
            'relations': tf.keras.losses.CategoricalCrossentropy(
                from_logits=True, name='rel_loss'),
        },
        metrics={
            'predictions': [
                MaskedSparseCategoricalAccuracy(
                    mask_id=0, from_logits=False, name='pred_acc'),
            ],
            'relations': [
                tf.keras.metrics.CategoricalAccuracy(name='rel_acc'),
            ]
        })
    model.summary()
    return model



import tensorflow as tf


class ScaledDotProductAttention(tf.keras.layers.Layer):

    def __init__(self, **kwargs):
        super(ScaledDotProductAttention, self).__init__(**kwargs)

    def call(self, inputs, training=None):
        query, key, value, mask = inputs
        score = tf.matmul(query, key, transpose_b=True)
        dk = tf.cast(tf.shape(query)[-1], tf.float32)
        score = score / tf.math.sqrt(dk)
        if mask is not None:
            score += mask * -10000.0
        attn_weights = tf.nn.softmax(score, axis=-1)
        context = tf.matmul(attn_weights, value)
        return context, attn_weights

    def get_config(self):
        return super().get_config()


class MultiHeadAttention(tf.keras.layers.Layer):

    def __init__(self, hidden_size=512, num_attention_heads=8, **kwargs):
        super(MultiHeadAttention, self).__init__(**kwargs)
        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        assert self.hidden_size % self.num_attention_heads == 0

        self.query_weight = tf.keras.layers.Dense(self.hidden_size, name='query')
        self.key_weight = tf.keras.layers.Dense(self.hidden_size, name='key')
        self.value_weight = tf.keras.layers.Dense(self.hidden_size, name='value')

        self.attention = ScaledDotProductAttention()

        self.dense = tf.keras.layers.Dense(self.hidden_size, name='dense')

    def _split_heads(self, x, batch_size):
        x = tf.reshape(x, (batch_size, -1, self.num_attention_heads, self.hidden_size // self.num_attention_heads))
        return tf.transpose(x, perm=[0, 2, 1, 3])

    def call(self, inputs, training=None):
        query, key, value, mask = inputs
        batch_size = tf.shape(query)[0]

        query = self._split_heads(self.query_weight(query), batch_size)
        key = self._split_heads(self.key_weight(key), batch_size)
        value = self._split_heads(self.value_weight(value), batch_size)

        context, attn_weights = self.attention(inputs=(query, key, value, mask))
        context = tf.transpose(context, perm=[0, 2, 1, 3])
        context = tf.reshape(context, [batch_size, -1, self.hidden_size])
        output = self.dense(context)
        return output, attn_weights

    def get_config(self):
        config = {
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class PointWiseFeedForwardNetwork(tf.keras.layers.Layer):

    def __init__(self, hidden_size=512, ffn_size=2048, **kwargs):
        super(PointWiseFeedForwardNetwork, self).__init__(**kwargs)
        self.ffn_size = ffn_size
        self.hidden_size = hidden_size
        self.dense1 = tf.keras.layers.Dense(self.ffn_size, activation='relu')
        self.dense2 = tf.keras.layers.Dense(self.hidden_size)

    def call(self, inputs, training=None):
        outputs = self.dense2(self.dense1(inputs))
        return outputs

    def get_config(self):
        config = {
            'ffn_size': self.ffn_size,
            'hidden_size': self.hidden_size,
        }
        p = super(PointWiseFeedForwardNetwork, self).get_config()
        return dict(list(p.items()) + list(config.items()))


class EncoderLayer(tf.keras.layers.Layer):

    def __init__(self, hidden_size=512, num_attention_heads=8, ffn_size=2048, dropout_rate=0.2, epsilon=1e-6, **kwargs):
        super(EncoderLayer, self).__init__(**kwargs)
        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        self.ffn_size = ffn_size
        self.dropout_rate = dropout_rate
        self.epsilon = epsilon

        self.attention = MultiHeadAttention(self.hidden_size, self.num_attention_heads)
        self.attn_dropout = tf.keras.layers.Dropout(self.dropout_rate)
        self.attn_layer_norm = tf.keras.layers.LayerNormalization(epsilon=self.epsilon)

        self.ffn = PointWiseFeedForwardNetwork(self.hidden_size, self.ffn_size)
        self.ffn_dropout = tf.keras.layers.Dropout(self.dropout_rate)
        self.ffn_layer_norm = tf.keras.layers.LayerNormalization(epsilon=self.epsilon)

    def call(self, inputs, training=None):
        query, key, value, mask = inputs
        attn, attn_weights = self.attention(inputs=(query, key, value, mask))
        attn = self.attn_dropout(attn, training=training)
        attn = self.attn_layer_norm(query + attn)

        ffn = self.ffn(attn)
        ffn = self.ffn_dropout(ffn, training=training)
        ffn = self.ffn_layer_norm(ffn + attn)

        return ffn, attn_weights

    def get_config(self):
        config = {
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads,
            'ffn_size': self.ffn_size,
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class DecoderLayer(tf.keras.layers.Layer):

    def __init__(self, hidden_size=512, num_attention_heads=8, ffn_size=2048, dropout_rate=0.2, epsilon=1e-6, **kwargs):
        super(DecoderLayer, self).__init__(**kwargs)
        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        self.ffn_size = ffn_size
        self.dropout_rate = dropout_rate
        self.epsilon = epsilon

        self.self_attention = MultiHeadAttention(self.hidden_size, self.num_attention_heads)
        self.self_attn_dropout = tf.keras.layers.Dropout(self.dropout_rate)
        self.self_attn_layer_norm = tf.keras.layers.LayerNormalization(epsilon=self.epsilon)

        self.context_attention = MultiHeadAttention(self.hidden_size, self.num_attention_heads)
        self.context_attn_dropout = tf.keras.layers.Dropout(self.dropout_rate)
        self.context_attn_layer_norm = tf.keras.layers.LayerNormalization(epsilon=self.epsilon)

        self.ffn = PointWiseFeedForwardNetwork(self.hidden_size, self.ffn_size)
        self.ffn_dropout = tf.keras.layers.Dropout(self.dropout_rate)
        self.ffn_layer_norm = tf.keras.layers.LayerNormalization(epsilon=self.epsilon)

    def call(self, inputs, training=None):
        x, enc_outputs, look_ahead_mask, padding_mask = inputs

        attn1, attn1_weights = self.self_attention(inputs=(x, x, x, look_ahead_mask))
        attn1 = self.self_attn_dropout(attn1, training=training)
        output1 = self.self_attn_layer_norm(attn1 + x)

        attn2, attn2_weights = self.context_attention(inputs=(output1, enc_outputs, enc_outputs, padding_mask))
        attn2 = self.context_attn_dropout(attn2, training=training)
        output2 = self.context_attn_layer_norm(attn2 + output1)

        ffn = self.ffn(output2)
        ffn = self.ffn_dropout(ffn, training=training)
        ffn = self.ffn_layer_norm(ffn + attn2)

        return ffn, attn1_weights, attn2_weights

    def get_config(self):
        config = {
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads,
            'ffn_size': self.ffn_size,
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon
        }
        base = super(DecoderLayer, self).get_config()
        return dict(list(base.items()) + list(config.items()))



import unittest

from .bert_adapter import BertAdapter


class BertAdapterTest(unittest.TestCase):

    def testLoadPretrainedBertModel(self):
        adapter = BertAdapter(strategy='chinese-bert-base')
        model, vocab_file = adapter.adapte('/Users/luozhouyang/pretrain-models/bert/chinese_L-12_H-768_A-12')

        print('model inputs: {}'.format(model.inputs))
        print('model outputs: {}'.format(model.outputs))


if __name__ == "__main__":
    unittest.main()



import json
import logging
import os

import tensorflow as tf

from transformers_keras import build_pretraining_bert_model
from transformers_keras.modeling_bert import Bert4PreTraining

from .abstract_adapter import AbstractAdapter, AbstractStrategy, PretrainedModelAdapter


def choose_strategy(strategy):
    if isinstance(strategy, AbstractStrategy):
        return strategy
    if 'chinese-bert-base' == strategy:
        return ChineseBertBaseStrategy()
    else:
        raise ValueError('Invalid strategy: {}'.format(strategy))


class ChineseBertBaseStrategy(AbstractStrategy):

    def mapping_config(self, pretrained_config_file):
        with open(pretrained_config_file, mode='rt', encoding='utf8') as fin:
            config = json.load(fin)

        model_config = {
            'vocab_size': config['vocab_size'],
            'activation': config['hidden_act'],
            'max_positions': config['max_position_embeddings'],
            'hidden_size': config['hidden_size'],
            'type_vocab_size': config['type_vocab_size'],
            'intermediate_size': config['intermediate_size'],
            'dropout_rate': config['hidden_dropout_prob'],
            'stddev': config['initializer_range'],
            'num_layers': config['num_hidden_layers'],
            'num_attention_heads': config['num_attention_heads'],
        }
        return model_config

    def build_model(self, model_config):
        model = build_pretraining_bert_model(model_config)
        return model

    def mapping_variables(self, model_config, model, ckpt):
        # model variable name -> pretrained bert variable name
        m = {
            'bert/main/embedding/weight:0': 'bert/embeddings/word_embeddings',
            'bert/main/embedding/position_embedding/embeddings:0': 'bert/embeddings/position_embeddings',
            'bert/main/embedding/token_type_embedding/embeddings:0': 'bert/embeddings/token_type_embeddings',
            'bert/main/embedding/layer_normalization/gamma:0': 'bert/embeddings/LayerNorm/gamma',
            'bert/main/embedding/layer_normalization/beta:0': 'bert/embeddings/LayerNorm/beta',
        }

        for i in range(model_config['num_layers']):
            # attention
            for n in ['query', 'key', 'value']:
                k = 'bert/main/encoder/layer_{}/mha/{}/kernel:0'.format(i, n)
                v = 'bert/encoder/layer_{}/attention/self/{}/kernel'.format(i, n)
                m[k] = v
                k = 'bert/main/encoder/layer_{}/mha/{}/bias:0'.format(i, n)
                v = 'bert/encoder/layer_{}/attention/self/{}/bias'.format(i, n)
                m[k] = v

            # dense after attention
            for n in ['kernel', 'bias']:
                k = 'bert/main/encoder/layer_{}/mha/dense/{}:0'.format(i, n)
                v = 'bert/encoder/layer_{}/attention/output/dense/{}'.format(i, n)
                m[k] = v
            # layer norm after attention
            for n in ['gamma', 'beta']:
                k = 'bert/main/encoder/layer_{}/attn_layer_norm/{}:0'.format(i, n)
                v = 'bert/encoder/layer_{}/output/LayerNorm/{}'.format(i, n)
                m[k] = v

            # intermediate
            for n in ['kernel', 'bias']:
                k = 'bert/main/encoder/layer_{}/intermediate/dense/{}:0'.format(i, n)
                v = 'bert/encoder/layer_{}/intermediate/dense/{}'.format(i, n)
                m[k] = v

            # output
            for n in ['kernel', 'bias']:
                k = 'bert/main/encoder/layer_{}/dense/{}:0'.format(i, n)
                v = 'bert/encoder/layer_{}/output/dense/{}'.format(i, n)
                m[k] = v

            # layer norm
            for n in ['gamma', 'beta']:
                k = 'bert/main/encoder/layer_{}/inter_layer_norm/{}:0'.format(i, n)
                v = 'bert/encoder/layer_{}/output/LayerNorm/{}'.format(i, n)
                m[k] = v

        # pooler
        for n in ['kernel', 'bias']:
            k = 'bert/main/pooler/dense/{}:0'.format(n)
            v = 'bert/pooler/dense/{}'.format(n)
            m[k] = v

        # masked lm
        m['bert/mlm/bias:0'] = 'cls/predictions/output_bias'
        for n in ['kernel', 'bias']:
            k = 'bert/mlm/dense/{}:0'.format(n)
            v = 'cls/predictions/transform/dense/{}'.format(n)
            m[k] = v
        for n in ['gamma', 'beta']:
            k = 'bert/mlm/layer_norm/{}:0'.format(n)
            v = 'cls/predictions/transform/LayerNorm/{}'.format(n)
            m[k] = v

        # nsp
        m['bert/nsp/dense/kernel:0'] = 'cls/seq_relationship/output_weights'
        m['bert/nsp/dense/bias:0'] = 'cls/seq_relationship/output_bias'

        return m

    def zip_weights(self, model, ckpt, variables_mapping):
        weights, values, names = [], [], []
        for w in model.trainable_weights:
            names.append(w.name)
            weights.append(w)
            v = tf.train.load_variable(ckpt, variables_mapping[w.name])
            if w.name == 'bert/nsp/dense/kernel:0':
                v = v.T
            values.append(v)

        logging.info('weights will be loadded from pretrained checkpoint: \n\t{}'.format('\n\t'.join(names)))

        mapped_values = zip(weights, values)
        return mapped_values


class BertAdapter(PretrainedModelAdapter):

    def __init__(self, strategy='chinese-bert-base'):
        self.strategy = choose_strategy(strategy)
        super(BertAdapter, self).__init__(strategy=self.strategy)



import tensorflow as tf
from tensorflow.python.ops import math_ops


def masked_sparse_categorical_crossentropy(y_true, y_pred):
    """Sparse categorical crossentropy with masking support for language models.

    Args:
        y_true: Tensor, shape (batch_size, time_steps)
        y_pred: Tensor, shape (batch_size, time_steps, vocab_size), outputs of softmax

    Returns:
        A loss scalar.
    """
    loss = tf.keras.losses.sparse_categorical_crossentropy(y_true, y_pred, from_logits=False, axis=-1)
    # masking positions where value equals 0 (usually padding token's id)
    mask = tf.logical_not(tf.equal(y_true, 0))  # shape (batch_size, time_steps)
    mask = tf.cast(mask, dtype=loss.dtype)
    numerator = tf.reduce_sum(loss * mask)  # total loss of valid positions
    denominator = tf.reduce_sum(mask) + 1e-6  # total number of valid position
    loss = numerator / denominator  # average loss
    return loss


class MaskedSparseCategoricalCrossentropy(tf.keras.losses.Loss):
    """Sparse categorical crossentropy with masking support."""

    def __init__(self, mask_id=0, from_logits=False, axis=-1, **kwargs):
        """Init.

        Args:
            mask_id: Python integer, value from `y_true` which needs to be masking.
            from_logits: Python boolean, `y_pred` is logits or not.
            axis: Python integer, which dimension to compute crossentropy
            epsilone: Python float, avoid divid by zero error
        """
        super().__init__(reduction='none', **kwargs)  # we do reduction mannually
        self.mask_id = mask_id
        self.from_logits = from_logits
        self.axis = axis

    def call(self, y_true, y_pred):
        loss = tf.keras.losses.sparse_categorical_crossentropy(
            y_true, y_pred, from_logits=self.from_logits, axis=self.axis)
        # masking positions where value equals `mask_id` (usually padding token's id)
        mask = tf.logical_not(tf.equal(y_true, self.mask_id))  # shape (batch_size, time_steps)
        mask = tf.cast(mask, dtype=loss.dtype)
        numerator = tf.reduce_sum(loss * mask)  # total loss of valid positions
        denominator = tf.reduce_sum(mask)  # total number of valid position
        loss = math_ops.div_no_nan(numerator, denominator)  # average loss over valid positions
        return loss



import numpy as np
import tensorflow as tf

from .modeling_bert import (
    Bert,
    Bert4PreTraining,
    BertEmbedding,
    BertEncoder,
    BertEncoderLayer,
    BertIntermediate,
    BertMLMHead,
    BertNSPHead,
    BertPooler,
)


class ModelingBertTest(tf.test.TestCase):

    def testBertEmbeddings(self):
        embedding_layer = BertEmbedding(vocab_size=16, max_positions=40)
        inputs = (
            tf.constant([[0, 2, 3, 4, 5, 1]]),
            # tf.constant([[0, 1, 2, 3, 4, 5]]),
            tf.constant([[0, 0, 0, 1, 1, 1]]),
        )
        embeddings = embedding_layer(inputs, mode='embedding', training=True)
        self.assertAllEqual([1, 6, embedding_layer.hidden_size], embeddings.shape)

        inputs2 = tf.random.uniform(shape=(1, embedding_layer.hidden_size))
        outputs = embedding_layer(inputs2, mode='linear')
        self.assertAllEqual([1, embedding_layer.vocab_size], outputs.shape)

    def testBertIntermediate(self):
        inter = BertIntermediate()
        inputs = tf.random.uniform(shape=(2, 16, 768))
        outputs = inter(inputs)
        self.assertAllEqual([2, 16, inter.intermediate_size], outputs.shape)

    def testBertEncoderLayer(self):
        encoder = BertEncoderLayer()
        hidden_states = tf.random.uniform((2, 10, encoder.hidden_size))
        attention_mask = None
        outputs, attention_weights = encoder(
            inputs=(hidden_states, attention_mask), training=True)

        self.assertAllEqual([2, 10, encoder.hidden_size], outputs.shape)
        self.assertAllEqual([2, encoder.num_attention_heads, 10, 10], attention_weights.shape)

    def testBertEncoder(self):
        encoder = BertEncoder(num_layers=2)

        hidden_states = tf.random.uniform((2, 10, encoder.hidden_size))
        attention_mask = None
        outputs, all_hidden_states, all_attention_scores = encoder(
            inputs=(hidden_states, attention_mask), training=True)

        self.assertAllEqual([2, 10, encoder.hidden_size], outputs.shape)

        self.assertAllEqual(encoder.num_layers, len(all_hidden_states))
        for state in all_hidden_states:
            self.assertAllEqual([2, 10, encoder.hidden_size], state.shape)

        self.assertAllEqual(encoder.num_layers, len(all_attention_scores))
        for attention in all_attention_scores:
            self.assertAllEqual([2, encoder.num_attention_heads, 10, 10], attention.shape)

    def testBertPooler(self):
        pooler = BertPooler()
        inputs = tf.random.uniform(shape=(2, 16, 768))
        outputs = pooler(inputs)
        self.assertAllEqual([2, pooler.hidden_size], outputs.shape)

    def testBertModel(self):
        model = Bert(vocab_size=100, num_layers=2)
        input_ids = tf.constant(
            [1, 2, 3, 4, 5, 6, 7, 5, 3, 2, 3, 4, 1, 2, 3, 1, 2, 3, 4, 5, 6, 6, 6, 7, 7, 8, 0, 0, 0, 0, 0, 0],
            shape=(2, 16),
            dtype=np.int32)  # input_ids
        token_type_ids = np.array(
            [[0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1],
             [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1]], dtype=np.int64)  # token_type_ids,
        input_mask = tf.constant(
            [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1],
             [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1]], dtype=np.float32)  # input_mask

        outputs, pooled_outputs, all_states, all_attn_weights = model(
            inputs=(input_ids, token_type_ids, input_mask))
        self.assertAllEqual([2, 16, model.hidden_size], outputs.shape)
        self.assertAllEqual([2, model.hidden_size], pooled_outputs.shape)

        self.assertEqual(2, len(all_states))
        for state in all_states:
            self.assertAllEqual([2, 16, model.hidden_size], state.shape)

        self.assertEqual(2, len(all_attn_weights))
        for attention in all_attn_weights:
            self.assertAllEqual([2, model.num_attention_heads, 16, 16], attention.shape)

    def testBertMLMHead(self):
        embedding = BertEmbedding(vocab_size=100)
        mlm = BertMLMHead(vocab_size=100, embedding=embedding)

        inputs = tf.random.uniform(shape=(2, 16, 768))
        outputs = mlm(inputs)
        self.assertAllEqual([2, 16, mlm.vocab_size], outputs.shape)

    def testBertNSPHead(self):
        nsp = BertNSPHead()
        inputs = tf.random.uniform(shape=(2, 768))
        outputs = nsp(inputs)
        self.assertAllEqual([2, 2], outputs.shape)

    def testBert4PreTraining(self):
        bert = Bert4PreTraining(vocab_size=100, num_layers=2)
        input_ids = tf.constant(
            [1, 2, 3, 4, 5, 6, 7, 5, 3, 2, 3, 4, 1, 2, 3, 1, 2, 3, 4, 5, 6, 6, 6, 7, 7, 8, 0, 0, 0, 0, 0, 0],
            shape=(2, 16),
            dtype=np.int32)  # input_ids
        token_type_ids = np.array(
            [[0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1],
             [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1]], dtype=np.int64)  # token_type_ids,
        input_mask = tf.constant(
            [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1],
             [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1]], dtype=np.float32)  # input_mask

        predictions, relations, attn_weights = bert(
            inputs=(input_ids, token_type_ids, input_mask))
        self.assertAllEqual([2, 16, 100], predictions.shape)
        self.assertAllEqual([2, 2], relations.shape)

        self.assertEqual(2, len(attn_weights))
        for attention in attn_weights:
            self.assertAllEqual([2, 8, 16, 16], attention.shape)


if __name__ == "__main__":
    tf.test.main()



import tensorflow as tf

from .layers import (DecoderLayer, EncoderLayer, MultiHeadAttention,
                     ScaledDotProductAttention)


def _create_padding_mask(x):
    mask = tf.cast(tf.equal(0, x), dtype=tf.float32)
    mask = mask[:, tf.newaxis, tf.newaxis, :]
    return mask


def _create_look_ahead_mask(size):
    mask = 1 - tf.linalg.band_part(tf.ones((size, size)), -1, 0)
    return mask


def _create_masks(x, y):
    _enc_padding_mask = _create_padding_mask(x)
    _dec_padding_mask = _create_padding_mask(x)
    _look_ahead_mask = _create_look_ahead_mask(tf.shape(y)[1])
    _target_padding_mask = _create_padding_mask(y)
    combined = tf.maximum(_look_ahead_mask, _target_padding_mask)
    return _enc_padding_mask, combined, _dec_padding_mask


class LayersTest(tf.test.TestCase):

    def testScaledDotProductAttention(self):
        attention = ScaledDotProductAttention()
        x = tf.random.uniform(shape=(4, 16, 64))
        mask = tf.cast(tf.linalg.band_part(
            tf.ones(shape=(4, 16, 16)), num_lower=0, num_upper=-1), dtype=tf.float32)
        context, attn_weight = attention(inputs=(x, x, x, mask), training=False)
        self.assertEqual([4, 16, 64], context.shape)
        self.assertEqual([4, 16, 16], attn_weight.shape)

    def testMultiHeadAttention(self):
        attention = MultiHeadAttention(hidden_size=512, num_attention_heads=8)
        x = tf.random.uniform(shape=(4, 16, 512))
        mask = tf.cast(tf.linalg.band_part(
            tf.ones(shape=(4, 1, 16, 16)), num_lower=0, num_upper=-1), dtype=tf.float32)
        context, attn_weight = attention(inputs=(x, x, x, mask), training=False)
        self.assertEqual([4, 16, 512], context.shape)
        self.assertEqual([4, 8, 16, 16], attn_weight.shape)

    def testEncoderLayer(self):
        encoder_layer = EncoderLayer(hidden_size=512, num_attention_heads=8, ffn_size=2048)
        x = tf.random.uniform(shape=(2, 5, 512))
        padding_mask = tf.constant([[0, 0, 0, 1, 1], [0, 0, 0, 0, 1]], shape=(2, 5), dtype=tf.float32)
        padding_mask = padding_mask[:, tf.newaxis, tf.newaxis, :]

        outputs, attn_weights = encoder_layer(inputs=(x, x, x, padding_mask))
        self.assertEqual([2, 5, 512], outputs.shape)
        self.assertEqual([2, 8, 5, 5], attn_weights.shape)

    def testDecoderLayer(self):
        decoder_layer = DecoderLayer(hidden_size=512, num_attention_heads=8, ffn_size=2048)
        x = tf.constant([[1, 2, 3, 4, 0], [2, 3, 3, 0, 0]], shape=(2, 5), dtype=tf.int64)
        y = tf.constant([[1, 2, 3, 4, 1, 0, 0], [2, 3, 3, 4, 5, 6, 7]], shape=(2, 7), dtype=tf.int64)

        _, look_ahead_mask, padding_mask = _create_masks(x, y)

        enc_outputs = tf.random.uniform(shape=(2, 5, 512))

        dec_inputs = tf.random.uniform((2, 7, 512))
        dec_outputs, attn_weights1, attn_weights2 = decoder_layer(
            inputs=(dec_inputs, enc_outputs, look_ahead_mask, padding_mask))

        self.assertEqual([2, 7, 512], dec_outputs.shape)
        self.assertEqual([2, 8, 7, 7], attn_weights1.shape)
        self.assertEqual([2, 8, 7, 5], attn_weights2.shape)


if __name__ == "__main__":
    tf.test.main()



import tensorflow as tf

from transformers_keras.tokenizers import TransformerDefaultTokenizer

from .transformer_dataset_builder import (TransformerTextFileDatasetBuilder,
                                          TransformerTFRecordDatasetBuilder)


class TransformerDatasetBuilderTest(tf.test.TestCase):

    def testTransformerTextFileDatasetBuilder(self):
        src_tokenizer = TransformerDefaultTokenizer(
            'testdata/vocab_src.txt',
            unk_token='<unk>', sos_token='<s>', eos_token='</s>', pad_token='<pad>')
        tgt_tokenizer = TransformerDefaultTokenizer(
            'testdata/vocab_tgt.txt',
            unk_token='<unk>', sos_token='<s>', eos_token='</s>', pad_token='<pad>')
        dataset_builder = TransformerTextFileDatasetBuilder(
            src_tokenizer, tgt_tokenizer,
            train_shuffle_buffer_size=100,
            valid_shuffle_buffer_size=100,
        )
        files = [('testdata/train.src.txt', 'testdata/train.tgt.txt')]

        train_dataset = dataset_builder.build_train_dataset(files)
        for d in train_dataset.take(2):
            print(d)

        valid_dataset = dataset_builder.build_valid_dataset(files)
        for d in valid_dataset.take(2):
            print(d)

    def testTransformerTFRecordDatasetBuilder(self):
        dataset_builder = TransformerTFRecordDatasetBuilder(
            src_max_len=16, tgt_max_len=16,
            train_shuffle_buffer_size=100,
            valid_shuffle_buffer_size=100,
        )

        files = ['testdata/transformer_train.tfrecord']

        train_dataset = dataset_builder.build_train_dataset(files)
        for d in train_dataset.take(2):
            print(d)

        valid_dataset = dataset_builder.build_valid_dataset(files)
        for d in valid_dataset.take(2):
            print(d)


if __name__ == "__main__":
    tf.test.main()



import tensorflow as tf

from .modeling_bert import BertEmbedding, BertEncoderLayer
from .modeling_utils import choose_activation, initialize


class AlbertEmbedding(BertEmbedding):

    def __init__(self,
                 vocab_size=-1, max_positions=512, embedding_size=128, type_vocab_size=2, dropout_rate=0.2,
                 stddev=0.02, epsilon=1e-12,
                 **kwargs):
        super().__init__(
            vocab_size=vocab_size,
            max_positions=max_positions, hidden_size=embedding_size, type_vocab_size=type_vocab_size,
            dropout_rate=dropout_rate, stddev=stddev, epsilon=epsilon,
            name='embedding',
            **kwargs)
        # embedding_size is not hidden_size in ALBERT
        self.embedding_size = embedding_size

    def get_config(self):
        config = {
            'embedding_size': self.embedding_size
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class AlbertEncoderLayer(BertEncoderLayer):

    def __init__(self,
                 hidden_size=768, num_attention_heads=8, intermediate_size=3072, activation='gelu',
                 dropout_rate=0.2, epsilon=1e-12, stddev=0.02,
                 **kwargs):
        super().__init__(
            hidden_size=hidden_size, num_attention_heads=num_attention_heads, intermediate_size=intermediate_size,
            activation=activation, dropout_rate=dropout_rate, epsilon=epsilon, stddev=stddev,
            **kwargs)

    def get_config(self):
        return super().get_config()


class AlbertEncoderGroup(tf.keras.layers.Layer):

    def __init__(self,
                 num_layers_each_group=1, hidden_size=768, num_attention_heads=8, intermediate_size=3072,
                 activation='gelu', dropout_rate=0.2, epsilon=1e-12, stddev=0.02,
                 **kwargs):
        super().__init__(**kwargs)
        self.num_layers_each_group = num_layers_each_group
        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        self.intermediate_size = intermediate_size
        self.activation = choose_activation(activation)
        self.dropout_rate = dropout_rate
        self.epsilon = epsilon
        self.stddev = stddev

        self.encoder_layers = [
            AlbertEncoderLayer(
                hidden_size=self.hidden_size,
                num_attention_heads=self.num_attention_heads,
                intermediate_size=self.intermediate_size,
                activation=self.activation,
                dropout_rate=self.dropout_rate,
                epsilon=self.epsilon,
                stddev=self.stddev,
                name='layer_{}'.format(i)
            ) for i in range(self.num_layers_each_group)
        ]

    def call(self, inputs, training=None):
        hidden_states, attn_mask = inputs

        group_hidden_states, group_attn_weights = [], []
        for idx, encoder in enumerate(self.encoder_layers):
            hidden_states, attn_weights = encoder(inputs=(hidden_states, attn_mask))
            group_hidden_states.append(hidden_states)
            group_attn_weights.append(attn_weights)

        return hidden_states, group_hidden_states, group_attn_weights

    def get_config(self):
        config = {
            'num_layers_each_group': self.num_layers_each_group,
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads,
            'intermediate_size': self.intermediate_size,
            'activation': tf.keras.activations.serialize(self.activation),
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon,
            'stddev': self.stddev,
        }
        base = super(AlbertEncoderGroup, self).get_config()
        return dict(list(base.items()) + list(config.items()))


class AlbertEncoder(tf.keras.layers.Layer):

    def __init__(self,
                 num_layers=12,
                 num_groups=1,
                 num_layers_each_group=1,
                 hidden_size=768,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 dropout_rate=0.2,
                 epsilon=1e-12,
                 stddev=0.02,
                 **kwargs):
        super(AlbertEncoder, self).__init__(name='encoder', **kwargs)
        self.num_layers = num_layers  # num of encoder layers
        self.num_groups = num_groups  # num of encoder groups
        self.num_layers_each_group = num_layers_each_group
        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        self.intermediate_size = intermediate_size
        self.activation = choose_activation(activation)
        self.dropout_rate = dropout_rate
        self.epsilon = epsilon
        self.stddev = stddev

        self.embedding_mapping = tf.keras.layers.Dense(
            self.hidden_size,
            kernel_initializer=initialize(self.stddev),
            name='embedding_mapping'
        )
        self.groups = [
            AlbertEncoderGroup(
                num_layers_each_group=self.num_layers_each_group,
                hidden_size=self.hidden_size,
                num_attention_heads=self.num_attention_heads,
                intermediate_size=self.intermediate_size,
                activation=self.activation,
                dropout_rate=self.dropout_rate,
                epsilon=self.epsilon,
                stddev=self.stddev,
                name='group_{}'.format(i),
                **kwargs) for i in range(self.num_groups)
        ]

    def call(self, inputs, training=None):
        hidden_states, attention_mask = inputs
        hidden_states = self.embedding_mapping(hidden_states)

        all_hidden_states, all_attn_weights = [], []
        for i in range(self.num_layers):
            layers_per_group = self.num_layers // self.num_groups
            group_index = i // layers_per_group
            hidden_states, group_hidden_states, group_attn_weights = self.groups[group_index](
                inputs=(hidden_states, attention_mask),
            )
            all_hidden_states.extend(group_hidden_states)
            all_attn_weights.extend(group_attn_weights)

        return hidden_states, all_hidden_states, all_attn_weights

    def get_config(self):
        config = {
            'num_layers': self.num_layers,
            'num_groups': self.num_groups,
            'num_layers_each_group': self.num_layers_each_group,
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads,
            'intermediate_size': self.intermediate_size,
            'activation': tf.keras.activations.serialize(self.activation),
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon,
            'stddev': self.stddev,
        }
        base = super(AlbertEncoder, self).get_config()
        return dict(list(base.items()) + list(config.items()))


class AlbertModel(tf.keras.layers.Layer):

    def __init__(self,
                 vocab_size=-1, max_positions=512, embedding_size=128, type_vocab_size=2, num_layers=12, num_groups=1,
                 num_layers_each_group=1, hidden_size=768, num_attention_heads=8, intermediate_size=3072,
                 activation='gelu', dropout_rate=0.2, epsilon=1e-12, stddev=0.02,
                 **kwargs):
        super(AlbertModel, self).__init__(name='main', **kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.max_positions = max_positions
        self.embedding_size = embedding_size
        self.type_vocab_size = type_vocab_size
        self.num_layers = num_layers  # num of encoder layers
        self.num_groups = num_groups  # num of encoder groups
        self.num_layers_each_group = num_layers_each_group
        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        self.intermediate_size = intermediate_size
        self.activation = choose_activation(activation)
        self.dropout_rate = dropout_rate
        self.epsilon = epsilon
        self.stddev = stddev

        self.embedding = AlbertEmbedding(
            vocab_size=self.vocab_size, max_positions=self.max_positions, embedding_size=self.embedding_size,
            type_vocab_size=self.type_vocab_size, dropout_rate=self.dropout_rate, epsilon=self.epsilon,
            stddev=self.stddev,
            **kwargs)

        self.encoder = AlbertEncoder(
            num_layers=self.num_layers, num_groups=self.num_groups, num_layers_each_group=self.num_layers_each_group,
            hidden_size=self.hidden_size, num_attention_heads=self.num_attention_heads,
            intermediate_size=self.intermediate_size, activation=self.activation, dropout_rate=dropout_rate,
            epsilon=self.epsilon, stddev=self.stddev,
            **kwargs)

        self.pooler = tf.keras.layers.Dense(
            self.hidden_size,
            kernel_initializer=initialize(self.stddev),
            activation='tanh',
            name='pooler'
        )

    def call(self, inputs, training=None):
        input_ids, segment_ids, mask = inputs
        mask = mask[:, tf.newaxis, tf.newaxis, :]
        embed = self.embedding(inputs=(input_ids, segment_ids), mode='embedding')
        outputs, all_hidden_states, all_attn_weights = self.encoder(inputs=(embed, mask))
        # take [CLS]
        pooled_output = self.pooler(outputs[:, 0])
        return outputs, pooled_output, all_hidden_states, all_attn_weights

    def get_config(self):
        config = {
            'vocab_size': self.vocab_size,
            'max_positions': self.max_positions,
            'hidden_size': self.hidden_size,
            'embedding_size': self.embedding_size,
            'type_vocab_size': self.type_vocab_size,
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon,
            'stddev': self.stddev,
            'num_layers': self.num_layers,
            'num_groups': self.num_groups,
            'num_layers_each_group': self.num_layers_each_group,
            'num_attention_heads': self.num_attention_heads,
            'intermediate_size': self.intermediate_size,
            'activation': tf.keras.activations.serialize(self.activation),
        }
        base = super(AlbertModel, self).get_config()
        return dict(list(base.items()) + list(config.items()))


class AlbertMLMHead(tf.keras.layers.Layer):

    def __init__(self, embedding, vocab_size=-1, activation='gelu', epsilon=1e-12, stddev=0.02, **kwargs):
        super(AlbertMLMHead, self).__init__(**kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.decoder = embedding  # use embedding matrix to decode
        self.activation = choose_activation(activation)
        self.stddev = stddev
        self.epsilon = epsilon
        self.dense = tf.keras.layers.Dense(
            self.decoder.embedding_size, kernel_initializer=initialize(self.stddev), name='dense')
        self.layer_norm = tf.keras.layers.LayerNormalization(epsilon=self.epsilon, name='layer_norm')

    def build(self, input_shape):
        self.decoder_bias = self.add_weight(
            shape=(self.vocab_size,),
            initializer='zeros',
            trainable=True,
            name='decoder/bias'
        )
        super().build(input_shape)

    def call(self, inputs, training=None):
        pooled_output = inputs
        output = self.layer_norm(self.activation(self.dense(pooled_output)))
        output = self.decoder(output, mode='linear') + self.decoder_bias
        return output

    def get_config(self):
        config = {
            'vocab_size': self.vocab_size,
            'activation': tf.keras.activations.serialize(self.activation),
            'stddev': self.stddev,
            'epsilon': self.epsilon
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class AlbertSOPHead(tf.keras.layers.Layer):

    def __init__(self, stddev=0.02, **kwargs):
        super(AlbertSOPHead, self).__init__(**kwargs)
        self.num_class = 2
        self.stddev = stddev
        self.classifier = tf.keras.layers.Dense(
            self.num_class, kernel_initializer=initialize(self.stddev), name='dense')

    def call(self, inputs, training=None):
        return self.classifier(inputs)

    def get_config(self):
        config = {
            'num_class': self.num_class,
            'stddev': self.stddev,
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))


class Albert4PreTraining(tf.keras.layers.Layer):

    def __init__(self,
                 vocab_size=-1, max_positions=512, embedding_size=128, type_vocab_size=2, num_layers=12,
                 num_groups=1, num_layers_each_group=1, hidden_size=768, num_attention_heads=8, intermediate_size=3072,
                 activation='gelu', dropout_rate=0.2, epsilon=1e-12, stddev=0.02, **kwargs):
        super().__init__(name='albert', **kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.max_positions = max_positions
        self.embedding_size = embedding_size
        self.type_vocab_size = type_vocab_size
        self.num_layers = num_layers  # num of encoder layers
        self.num_groups = num_groups  # num of encoder groups
        self.num_layers_each_group = num_layers_each_group
        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        self.intermediate_size = intermediate_size
        self.activation = choose_activation(activation)
        self.dropout_rate = dropout_rate
        self.epsilon = epsilon
        self.stddev = stddev

        self.albert = AlbertModel(
            vocab_size, max_positions=self.max_positions, embedding_size=self.embedding_size,
            type_vocab_size=self.type_vocab_size, num_layers=self.num_layers, num_groups=self.num_groups,
            num_layers_each_group=self.num_layers_each_group, hidden_size=self.hidden_size,
            num_attention_heads=self.num_attention_heads, intermediate_size=self.intermediate_size,
            activation=self.activation, dropout_rate=self.dropout_rate, epsilon=self.epsilon, stddev=self.stddev,
            **kwargs)
        self.mlm = AlbertMLMHead(
            vocab_size=self.vocab_size, embedding=self.albert.embedding, activation=self.activation,
            epsilon=self.epsilon, stddev=self.stddev,
            name='mlm',
            **kwargs)
        self.sop = AlbertSOPHead(stddev=self.stddev, name='sop', **kwargs)

    def call(self, inputs, training=None):
        input_ids, segment_ids, mask = inputs
        mask = tf.cast(mask, dtype=tf.float32)
        outputs, pooled_outputs, all_hidden_states, all_attn_weights = self.albert(
            inputs=(input_ids, segment_ids, mask))
        mlm_output = self.mlm(outputs)
        sop_output = self.sop(pooled_outputs)
        return mlm_output, sop_output, all_hidden_states, all_attn_weights

    def get_config(self):
        config = {
            'vocab_size': self.vocab_size,
            'max_positions': self.max_positions,
            'hidden_size': self.hidden_size,
            'embedding_size': self.embedding_size,
            'type_vocab_size': self.type_vocab_size,
            'dropout_rate': self.dropout_rate,
            'epsilon': self.epsilon,
            'stddev': self.stddev,
            'num_layers': self.num_layers,
            'num_groups': self.num_groups,
            'num_layers_each_group': self.num_layers_each_group,
            'num_attention_heads': self.num_attention_heads,
            'intermediate_size': self.intermediate_size,
            'activation': tf.keras.activations.serialize(self.activation),
        }
        base = super().get_config()
        return dict(list(base.items()) + list(config.items()))



import abc
import logging
import os

import tensorflow as tf


class AbstractStrategy(abc.ABC):
    """Pretrained model load strategy."""

    def mapping_config(self, pretrained_config_file):
        """Convert pretrained configs to model configs.

        Args:
            pretrained_config_file: File path of pretrained model's config

        Returns:
            A python dict, model config
        """
        raise NotImplementedError()

    def build_model(self, model_config):
        """Build and compile model accroding to model config.

        Args:
            model_config: A python dict, model's config

        Returns:
            A keras model, instance of `tf.keras.Model`, compiled.
        """
        raise NotImplementedError()

    def mapping_variables(self, model_config, model, ckpt):
        """Mapping pretrained variables to model's variables.

        Args:
            model_config: A python dict, model's config
            model: A keras model, compiled
            ckpt: Python str, pretrained checkpoint model.

        Returns:
            Python dict, variables' name mapping
        """
        raise NotImplementedError()

    def zip_weights(self, model, ckpt, variables_mapping):
        """Zip weights and values.

        Args:
            model: A keras model, compiled
            ckpt: Python str, pretrained checkpoint model
            variables_mapping: Python dict, variables' name mapping

        Returns:
            A List of tuple (model_weight, pretrained_weight)
        """
        raise NotImplementedError()


class AbstractAdapter(abc.ABC):

    @abc.abstractmethod
    def adapte(self, pretrain_model_dir, **kwargs):
        raise NotImplementedError()

    def _parse_files(self, pretrain_model_dir):
        config_file, ckpt, vocab = None, None, None
        if not os.path.exists(pretrain_model_dir):
            logging.info('pretrain model dir: {} is not exists.'.format(pretrain_model_dir))
            return
        for f in os.listdir(pretrain_model_dir):
            if str(f).endswith('config.json'):
                config_file = os.path.join(pretrain_model_dir, f)
            if 'vocab' in str(f):
                vocab = os.path.join(pretrain_model_dir, f)
            if 'ckpt' in str(f):
                n = '.'.join(str(f).split('.')[:-1])
                ckpt = os.path.join(pretrain_model_dir, n)
        return config_file, ckpt, vocab


class PretrainedModelAdapter(AbstractAdapter):
    """Base class of pretrain models' adapter."""

    def __init__(self, strategy: AbstractStrategy):
        """Init.

        Args:
            strategy: An instance of `AbstractStrategy`
        """
        super().__init__()
        self.strategy = strategy

    def adapte(self, pretrain_model_dir, **kwargs):
        config_file, ckpt, vocab_file = self._parse_files(pretrain_model_dir)
        model_config = self.strategy.mapping_config(config_file)
        model = self.strategy.build_model(model_config)
        names_mapping = self.strategy.mapping_variables(model_config, model, ckpt)
        weights_values = self.strategy.zip_weights(model, ckpt, names_mapping)
        tf.keras.backend.batch_set_value(weights_values)
        return model, vocab_file



from .abstract_adapter import AbstractAdapter, AbstractStrategy, PretrainedModelAdapter
from .albert_adapter import AlbertAdapter, ChineseAlbertLargeStrategy
from .bert_adapter import BertAdapter, ChineseBertBaseStrategy



import abc

from .tokenizer import BasicTokenizer, WordpieceTokenizer


class BertAbstractTokenizer(abc.ABC):

    def __init__(self,
                 pad_token='[PAD]',
                 unk_token='[UNK]',
                 sos_token='[SOS]',
                 eos_token='[EOS]',
                 cls_token='[CLS]',
                 sep_token='[SEP]',
                 mask_token='[MASK]',
                 **kwargs):
        super().__init__()
        self.pad_token = pad_token
        self.unk_token = unk_token
        self.sos_token = sos_token
        self.eos_token = eos_token
        self.cls_token = cls_token
        self.sep_token = sep_token
        self.mask_token = mask_token

    def tokenize(self, sequence):
        raise NotImplementedError()

    def encode(self, sequence):
        raise NotImplementedError()

    def decode(self, sequence):
        raise NotImplementedError()

    @property
    def vocab_size(self):
        raise NotImplementedError()

    @property
    def special_tokens(self):
        return [self.pad_token, self.unk_token, self.sos_token, self.eos_token,
                self.cls_token, self.sep_token, self.mask_token]


class BertVocabBasedTokenizer(BertAbstractTokenizer):
    """Tokenizer that based on a vocab file. All special tokens will be moved to the head of the file."""

    def __init__(self, vocab_file, **kwargs):
        super(BertVocabBasedTokenizer, self).__init__(**kwargs)
        # vocabs = special tokens + vocab tokens, move special tokens to the head of list
        self.vocab = self.special_tokens + self._load_vocab(vocab_file)
        self.token2id = self._build_index(self.vocab)
        self.id2token = self._build_reverse_index(self.token2id)
        self.pad_id = self.token2id[self.pad_token]
        self.unk_id = self.token2id[self.unk_token]
        self.sos_id = self.token2id[self.sos_token]
        self.eos_id = self.token2id[self.eos_token]
        self.cls_id = self.token2id[self.cls_token]
        self.sep_id = self.token2id[self.sep_token]
        self.mask_id = self.token2id[self.mask_token]

    def _load_vocab(self, file):
        vocabs = []
        with open(file, mode='rt', encoding='utf8') as fin:
            for line in fin:
                word = line.strip('\n').strip()
                if not word:
                    continue
                if word in self.special_tokens:
                    continue
                vocabs.append(word)
        return vocabs

    def _build_index(self, vocab):
        m = {}
        for i, v in enumerate(vocab):
            m[v] = i
        return m

    def _build_reverse_index(self, token2id):
        m = {}
        for k, v in token2id.items():
            m[v] = k
        return m

    @property
    def vocab_size(self):
        return len(self.token2id)

    def tokenize(self, sequence):
        raise NotImplementedError()

    def encode(self, sequence):
        if isinstance(sequence, str):
            tokens = self.tokenize(sequence)
            return [self.token2id.get(t, self.unk_id) for t in tokens]
        elif isinstance(sequence, list):
            return [self.token2id.get(t, self.unk_id) for t in sequence]
        else:
            raise ValueError('Invalid `sequence` type. Must be one of [str, list]')

    def decode(self, sequence):
        return [self.id2token.get(_id, self.unk_id) for _id in sequence]


class BertDefaultTokenizer(BertVocabBasedTokenizer):
    """Default tokenizer, support basic tokenizer and wordpiece tokenizer."""

    def __init__(self, vocab_file, **kwargs):
        super().__init__(vocab_file, **kwargs)
        self.do_basic_tokenization = kwargs.get('do_basic_tokenization', True)
        self.do_lower_case = kwargs.get('do_lower_case', True)
        self.never_split = kwargs.get('never_split', None)
        self.tokenize_chinese_chars = kwargs.get('tokenize_chinese_chars', True)
        if self.do_basic_tokenization:
            self.basic_tokenizer = BasicTokenizer(
                do_lower_case=self.do_lower_case,
                never_split=self.never_split,
                tokenize_chinese_chars=self.tokenize_chinese_chars
            )
        else:
            self.basic_tokenizer = None

        self.max_input_chars_per_word = kwargs.get('max_input_chars_per_word', 100)
        self.wordpiece_tokenizer = WordpieceTokenizer(
            vocab=self.vocab,
            unk_token=self.unk_token,
            max_input_chars_per_word=self.max_input_chars_per_word
        )

    def tokenize(self, sequence):
        tokens = []
        if self.do_basic_tokenization:
            for token in self.basic_tokenizer.tokenize(sequence, never_split=self.never_split):
                for t in self.wordpiece_tokenizer.tokenize(token):
                    tokens.append(t)
        else:
            tokens = self.wordpiece_tokenizer.tokenize(sequence)
        return tokens



import collections
import logging
import os

import tensorflow as tf

from transformers_keras.tokenizers import TransformerDefaultTokenizer


def create_int_feature(values):
    feature = tf.train.Feature(int64_list=tf.train.Int64List(value=list(values)))
    return feature


class TransformerTFRecordGenerator(object):

    def __init__(self,
                 src_tokenizer,
                 tgt_tokenizer,
                 src_max_len=512,
                 tgt_max_len=512,
                 record_option='GZIP',
                 log_steps=1000,
                 **kwargs):
        super().__init__()
        self.src_tokenizer = src_tokenizer
        self.tgt_tokenizer = tgt_tokenizer

        self.src_max_len = src_max_len
        self.tgt_max_len = tgt_max_len

        self.record_option = record_option
        self.log_steps = log_steps

    def generate(self, input_files, output_files):
        writers = []
        for output_file in output_files:
            writers.append(tf.io.TFRecordWriter(output_file, options=self.record_option))

        total = 0
        writer_index = 0
        for input_file in input_files:
            if not os.path.exists(input_file):
                logging.warning('File %s does not exists.' % input_file)
                continue
            with open(input_file, mode='rt', encoding='utf8') as fin:
                while True:
                    line = fin.readline()
                    if not line:
                        break
                    src, tgt = self._parse_line(line)
                    if not src or not tgt:
                        continue

                    src_ids = self.src_tokenizer.encode(src)
                    src_ids = [self.src_tokenizer.sos_id] + src_ids + [self.src_tokenizer.eos_id]
                    while len(src_ids) < self.src_max_len:
                        src_ids.append(self.src_tokenizer.pad_id)

                    if len(src_ids) > self.src_max_len:
                        logging.warning('Length of sequence is {}, greater than {}', len(src_ids), self.src_max_len)
                        continue

                    tgt_ids = self.tgt_tokenizer.encode(tgt)
                    tgt_ids = [self.tgt_tokenizer.sos_id] + tgt_ids + [self.tgt_tokenizer.eos_id]
                    while len(tgt_ids) < self.tgt_max_len:
                        tgt_ids.append(self.tgt_tokenizer.pad_id)
                    if len(tgt_ids) > self.tgt_max_len:
                        logging.warning('Length of sequence is {}, greater than {}', len(tgt_ids), self.tgt_max_len)
                        continue

                    instance = {
                        # 'src_tokens': src_tokens,
                        'src_ids': src_ids,
                        # 'tgt_tokens': tgt_tokens,
                        'tgt_ids': tgt_ids
                    }

                    try:
                        example = self._create_example(instance)
                        if example is not None:
                            writers[writer_index].write(example.SerializeToString())
                            total += 1
                    except Exception as e:
                        logging.warning('Write example exception: %s' % e)
                        continue

                    if total % self.log_steps == 0:
                        logging.info('Write %d examples.' % total)
                    writer_index = (writer_index + 1) % len(writers)

        logging.info('Write %d examples in total.' % total)
        for writer in writers:
            writer.close()

    def _parse_line(self, line):
        parts = line.split('@')
        if len(parts) != 2:
            return '', ''
        src, tgt = parts[0].strip(), parts[1].strip()
        return src, tgt

    def _create_example(self, instance):
        features = collections.OrderedDict()
        features['src_ids'] = create_int_feature(instance['src_ids'])
        features['tgt_ids'] = create_int_feature(instance['tgt_ids'])
        example = tf.train.Example(features=tf.train.Features(feature=features))
        return example



import logging
import os

import tensorflow as tf

from .callbacks import TransformerLearningRate, SavedModelExporter
from .datasets import AbstractDatasetBuilder
from .losses import MaskedSparseCategoricalCrossentropy
from .metrics import MaskedSparseCategoricalAccuracy
from .modeling_bert import Bert4PreTraining
from .modeling_transformer import Transformer
from .modeling_albert import Albert4PreTraining


class AbstractRunner(object):

    def __init__(self, model_config, dataset_builder: AbstractDatasetBuilder, model_dir='/tmp/model', **kwargs):
        super().__init__()
        self.model_config = model_config
        self.dataset_builder = dataset_builder
        self.model_dir = model_dir
        if not os.path.exists(self.model_dir):
            os.makedirs(self.model_dir)
        self.tensorboard_dir = os.path.join(self.model_dir, 'tensorboard')
        self.export_dir = os.path.join(self.model_dir, 'export')
        self.model = self._build_model(self.model_config)
        logging.info('Models will be saved to {}.'.format(self.model_dir))
        logging.info('Tensorboard logs will be saved to {}.'.format(self.tensorboard_dir))
        logging.info('Exported models will be saved to {}.'.format(self.export_dir))

    def _build_callbacks(self, callbacks=None, ckpt_steps=2000, export_steps=5000, **kwargs):
        train_callbacks = [
            tf.keras.callbacks.EarlyStopping(monitor='val_loss', patience=5),
            tf.keras.callbacks.TensorBoard(self.tensorboard_dir),
            tf.keras.callbacks.ModelCheckpoint(
                filepath=self.model_dir,
                save_weights_only=True,
                save_freq=self.dataset_builder.train_batch_size * ckpt_steps
            ),
            SavedModelExporter(self.export_dir, every_steps=export_steps),
        ]
        if callbacks:
            train_callbacks = train_callbacks + callbacks
        return train_callbacks

    def train(self, *args, **kwargs):
        raise NotImplementedError()

    def evaluate(self, *args, **kwargs):
        raise NotImplementedError()

    def predict(self, *args, **kwargs):
        raise NotImplementedError()

    def _build_model(self, config):
        raise NotImplementedError()


class TransformerRunner(AbstractRunner):

    def __init__(self,
                 model_config,
                 dataset_builder: AbstractDatasetBuilder,
                 model_dir='/tmp/transformer',
                 **kwargs):
        super(TransformerRunner, self).__init__(model_config, dataset_builder, model_dir, **kwargs)

    def _build_model(self, config):
        x = tf.keras.layers.Input(shape=(None,), dtype=tf.int32, name='x')
        y = tf.keras.layers.Input(shape=(None,), dtype=tf.int32, name='y')
        model = Transformer(**config)
        logits, _, _, _ = model(inputs=(x, y))
        probs = tf.keras.layers.Lambda(
            lambda x: tf.nn.softmax(x), name='probs')(logits)

        model = tf.keras.Model(inputs=[x, y], outputs=[probs])

        lr = TransformerLearningRate(config.get('hidden_size', 512))
        model.compile(
            optimizer=tf.keras.optimizers.Adam(lr, beta_1=0.9, beta_2=0.98, epsilon=1e-9),
            loss=tf.keras.losses.SparseCategoricalCrossentropy(name='loss', from_logits=False),
            metrics=[tf.keras.metrics.SparseCategoricalAccuracy(name='acc')],
        )
        model.summary()
        return model

    def train(self,
              train_files,
              valid_files=None,
              epochs=10,
              ckpt_steps=2000,
              export_steps=5000,
              callbacks=None,
              **kwargs):
        valid_dataset = self.dataset_builder.build_valid_dataset(valid_files) if valid_files else None
        train_dataset = self.dataset_builder.build_train_dataset(train_files)

        callbacks = self._build_callbacks(callbacks, ckpt_steps=ckpt_steps, export_steps=export_steps, **kwargs)

        history = self.model.fit(
            train_dataset,
            validation_data=valid_dataset,
            epochs=epochs,
            callbacks=callbacks,
            **kwargs
        )
        return history

    def evaluate(self, valid_files, **kwargs):
        pass

    def predict(self, input_files, **kwargs):
        pass


class BertRunner(TransformerRunner):

    def __init__(self,
                 model_config,
                 dataset_builder: AbstractDatasetBuilder,
                 model_dir='/tmp/bert',
                 **kwargs):
        super().__init__(model_config, dataset_builder, model_dir=model_dir, **kwargs)

    def _build_model(self, config):
        max_sequence_length = config.get('max_positions', 512)
        input_ids = tf.keras.layers.Input(
            shape=(max_sequence_length,), dtype=tf.int32, name='input_ids')
        input_mask = tf.keras.layers.Input(
            shape=(max_sequence_length,), dtype=tf.int32, name='input_mask')
        segment_ids = tf.keras.layers.Input(
            shape=(max_sequence_length,), dtype=tf.int32, name='segment_ids')

        inputs = (input_ids, segment_ids, input_mask)
        outputs = Bert4PreTraining(**config, name='bert')(inputs=inputs)

        predictions = tf.keras.layers.Lambda(
            lambda x: x, name='predictions')(outputs[0])
        relations = tf.keras.layers.Lambda(
            lambda x: tf.nn.softmax(x), name='relations')(outputs[1])
        # attentions = tf.keras.layers.Lambda(
        #   lambda x: x, name='attentions')(outputs[2])

        model = tf.keras.Model(
            inputs=[input_ids, segment_ids, input_mask], outputs=[predictions, relations])
        model.compile(
            optimizer=tf.keras.optimizers.Adam(learning_rate=3e-5, epsilon=1e-12, clipnorm=1.0),
            loss={
                'predictions': MaskedSparseCategoricalCrossentropy(
                    mask_id=0, from_logits=True, name='pred_loss'),
                'relations': tf.keras.losses.CategoricalCrossentropy(
                    from_logits=True, name='rel_loss'),
            },
            metrics={
                'predictions': [
                    MaskedSparseCategoricalAccuracy(
                        mask_id=0, from_logits=False, name='pred_acc'),
                ],
                'relations': [
                    tf.keras.metrics.CategoricalAccuracy(name='rel_acc'),
                ]
            })
        model.summary()
        return model


class AlbertRunner(TransformerRunner):

    def __init__(self, model_config, dataset_builder, model_dir='/tmp/albert', **kwargs):
        super().__init__(model_config, dataset_builder, model_dir=model_dir, **kwargs)

    def _build_model(self, config):
        max_sequence_length = config.get('max_positions', 512)
        input_ids = tf.keras.layers.Input(
            shape=(max_sequence_length,), dtype=tf.int32, name='input_ids')
        input_mask = tf.keras.layers.Input(
            shape=(max_sequence_length,), dtype=tf.int32, name='input_mask')
        segment_ids = tf.keras.layers.Input(
            shape=(max_sequence_length,), dtype=tf.int32, name='segment_ids')

        inputs = (input_ids, segment_ids, input_mask)
        albert = Albert4PreTraining(**config)
        predictions, relations, all_states, all_attn_weights = albert(inputs=inputs)

        predictions = tf.keras.layers.Lambda(lambda x: x, name='predictions')(predictions)
        relations = tf.keras.layers.Lambda(lambda x: x, name='relations')(relations)

        model = tf.keras.Model(
            inputs=[input_ids, segment_ids, input_mask], outputs=[predictions, relations])

        model.compile(
            optimizer=tf.keras.optimizers.Adam(learning_rate=3e-5, epsilon=1e-12, clipnorm=1.0),
            loss={
                'predictions': MaskedSparseCategoricalCrossentropy(
                    mask_id=0, from_logits=True, name='pred_loss'),
                'relations': tf.keras.losses.CategoricalCrossentropy(
                    from_logits=True, name='rel_loss'),
            },
            metrics={
                'predictions': [
                    MaskedSparseCategoricalAccuracy(
                        mask_id=0, from_logits=False, name='pred_acc'),
                ],
                'relations': [
                    tf.keras.metrics.CategoricalAccuracy(name='rel_acc'),
                ]
            })
        model.summary()
        return model



import json
import logging
import os

import tensorflow as tf

from transformers_keras import build_pretraining_albert_model

from .abstract_adapter import AbstractAdapter, AbstractStrategy, PretrainedModelAdapter


def choose_strategy(strategy):
    if isinstance(strategy, AbstractStrategy):
        return strategy
    if 'zh-albert-large' == strategy:
        return ChineseAlbertLargeStrategy()
    else:
        raise ValueError('Invalid strategy: {}'.format(strategy))


class ChineseAlbertLargeStrategy(AbstractStrategy):

    def mapping_config(self, pretrained_config_file):
        with open(pretrained_config_file, mode='rt', encoding='utf8') as fin:
            config = json.load(fin)

        model_config = {
            'vocab_size': config['vocab_size'],
            'max_positions': config['max_position_embeddings'],
            'embedding_size': config['embedding_size'],
            'type_vocab_size': config['type_vocab_size'],
            'num_layers': config['num_hidden_layers'],
            'num_groups': config['num_hidden_groups'],
            'num_layers_each_group': config['inner_group_num'],
            'hidden_size': config['hidden_size'],
            'num_attention_heads': config['num_attention_heads'],
            'intermediate_size': config['intermediate_size'],
            'activation': config['hidden_act'],
            'dropout_rate': config['hidden_dropout_prob'],
            'stddev': config['initializer_range'],
        }
        return model_config

    def build_model(self, model_config):
        return build_pretraining_albert_model(model_config)

    def mapping_variables(self, model_config, model, ckpt):

        model_names = []
        for w in model.trainable_weights:
            model_names.append(w.name)

        logging.info('Model trainable weights: \n\t{}'.format('\n\t'.join(model_names)))
        pretrained_names = [v[0] for v in tf.train.list_variables(ckpt)]
        logging.info('Pretrained  model trainable weights: \n\t{}'.format('\n\t'.join(pretrained_names)))

        logging.info('model weights num: {}, pretrained model weights num: {}'.format(
            len(model.trainable_weights), len(pretrained_names)))

        m = {
            'albert/main/embedding/weight:0': 'bert/embeddings/word_embeddings',
            'albert/main/embedding/position_embedding/embeddings:0': 'bert/embeddings/position_embeddings',
            'albert/main/embedding/token_type_embedding/embeddings:0': 'bert/embeddings/token_type_embeddings',
            'albert/main/embedding/layer_norm/gamma:0': 'bert/embeddings/LayerNorm/gamma',
            'albert/main/embedding/layer_norm/beta:0': 'bert/embeddings/LayerNorm/beta'
        }

        for n in ['kernel', 'bias']:
            k = 'albert/main/encoder/embedding_mapping/{}:0'.format(n)
            v = 'bert/encoder/embedding_hidden_mapping_in/{}'.format(n)
            m[k] = v

        for group in range(model_config['num_groups']):
            for layer in range(model_config['num_layers_each_group']):
                k_prefix = 'albert/main/encoder/group_{}/layer_{}/'.format(group, layer)
                v_prefix = 'bert/encoder/transformer/group_{}/inner_group_{}/'.format(group, layer)

                # attention
                for n in ['query', 'key', 'value']:
                    for x in ['kernel', 'bias']:
                        k = k_prefix + 'mha/{}/{}:0'.format(n, x)
                        v = v_prefix + 'attention_1/self/{}/{}'.format(n, x)
                        m[k] = v

                # attention dense
                for n in ['kernel', 'bias']:
                    k = k_prefix + 'mha/dense/{}:0'.format(n)
                    v = v_prefix + 'attention_1/output/dense/{}'.format(n)
                    m[k] = v

                # attention layer norm
                for n in ['gamma', 'beta']:
                    k = k_prefix + 'attn_layer_norm/{}:0'.format(n)
                    v = v_prefix + 'LayerNorm/{}'.format(n)
                    m[k] = v

                # intermediate
                for n in ['kernel', 'bias']:
                    k = k_prefix + 'intermediate/dense/{}:0'.format(n)
                    v = v_prefix + 'ffn_1/intermediate/dense/{}'.format(n)
                    m[k] = v
                    k = k_prefix + 'dense/{}:0'.format(n)
                    v = v_prefix + 'ffn_1/intermediate/output/dense/{}'.format(n)
                    m[k] = v

                # layer norm
                for n in ['gamma', 'beta']:
                    k = k_prefix + 'inter_layer_norm/{}:0'.format(n)
                    v = v_prefix + 'LayerNorm_1/{}'.format(n)
                    m[k] = v

        # pooler
        for n in ['kernel', 'bias']:
            k = 'albert/main/pooler/{}:0'.format(n)
            v = 'bert/pooler/dense/{}'.format(n)
            m[k] = v

        # mlm
        for n in ['kernel', 'bias']:
            k = 'albert/mlm/dense/{}:0'.format(n)
            v = 'cls/predictions/transform/dense/{}'.format(n)
            m[k] = v

        for n in ['gamma', 'beta']:
            k = 'albert/mlm/layer_norm/{}:0'.format(n)
            v = 'cls/predictions/transform/LayerNorm/{}'.format(n)
            m[k] = v

        m['albert/mlm/decoder/bias:0'] = 'cls/predictions/output_bias'
        m['albert/sop/dense/kernel:0'] = 'cls/seq_relationship/output_weights'
        m['albert/sop/dense/bias:0'] = 'cls/seq_relationship/output_bias'

        return m

    def zip_weights(self, model, ckpt, variables_mapping):
        weights, values, names = [], [], []
        for w in model.trainable_weights:
            names.append(w.name)
            weights.append(w)
            v = tf.train.load_variable(ckpt, variables_mapping[w.name])
            if w.name == 'albert/sop/dense/kernel:0':
                v = v.T
            values.append(v)

        logging.info('weights will be loadded from pretrained checkpoint: \n\t{}'.format('\n\t'.join(names)))

        mapped_values = zip(weights, values)
        return mapped_values


class AlbertAdapter(PretrainedModelAdapter):

    def __init__(self, strategy='zh-albert-large'):
        self.strategy = choose_strategy(strategy)
        super(AlbertAdapter, self).__init__(strategy=self.strategy)



import unittest

from .transformer_tokenizer import TransformerDefaultTokenizer
from .transformer_tokenizer import TransformerJiebaTokenizer


class TransformerTokenizerTest(unittest.TestCase):

    def testTransformerDefaultTokenizer(self):
        tokenizer = TransformerDefaultTokenizer(
            'testdata/vocab_src.txt',
            unk_token='<unk>', sos_token='<s>', eos_token='</s>', pad_token='<pad>')
        # print(tokenizer.token2id)
        self.assertEqual(0, tokenizer.pad_id)
        self.assertEqual(1, tokenizer.unk_id)
        self.assertEqual(2, tokenizer.sos_id)
        self.assertEqual(3, tokenizer.eos_id)
        self.assertEqual('<pad>', tokenizer.pad_token)
        self.assertEqual('<unk>', tokenizer.unk_token)
        self.assertEqual('<s>', tokenizer.sos_token)
        self.assertEqual('</s>', tokenizer.eos_token)

        seq = '我 住在 上海'
        self.assertEqual(['我', '住', '在', '上', '海'], tokenizer.tokenize(seq))
        self.assertEqual([4, 5, 6, 7, 8], tokenizer.encode(seq))

    def testTransformerJiebaTokenizer(self):
        tokenizer = TransformerJiebaTokenizer(
            'testdata/vocab_src.txt',
            unk_token='<unk>', sos_token='<s>', eos_token='</s>', pad_token='<pad>')
        print(tokenizer.token2id)
        self.assertEqual(0, tokenizer.pad_id)
        self.assertEqual(1, tokenizer.unk_id)
        self.assertEqual(2, tokenizer.sos_id)
        self.assertEqual(3, tokenizer.eos_id)
        self.assertEqual('<pad>', tokenizer.pad_token)
        self.assertEqual('<unk>', tokenizer.unk_token)
        self.assertEqual('<s>', tokenizer.sos_token)
        self.assertEqual('</s>', tokenizer.eos_token)

        seq = '我 住在 上海'
        self.assertEqual(['我', '住', '在', '上海'], tokenizer.tokenize(seq))
        self.assertEqual([4, 5, 6, 1], tokenizer.encode(seq))


if __name__ == "__main__":
    unittest.main()



import unittest

import tensorflow as tf

from .albert_adapter import AlbertAdapter, ChineseAlbertLargeStrategy


class AlbertAdapterTest(unittest.TestCase):

    def testAlbertAdapter(self):
        adapter = AlbertAdapter('zh-albert-large')
        model, vocab = adapter.adapte('/Users/luozhouyang/pretrain-models/albert/albert_large')


if __name__ == "__main__":
    unittest.main()



import tensorflow as tf

from transformers_keras import (TransformerDefaultTokenizer, TransformerRunner,
                                TransformerTextFileDatasetBuilder)
from transformers_keras import BertTFRecordDatasetBuilder
from transformers_keras import BertRunner
from transformers_keras import AlbertRunner


class RunnersTest(tf.test.TestCase):

    def testTransformer(self):

        src_tokenizer = TransformerDefaultTokenizer(vocab_file='testdata/vocab_src.txt')
        tgt_tokenizer = TransformerDefaultTokenizer(vocab_file='testdata/vocab_tgt.txt')
        dataset_builder = TransformerTextFileDatasetBuilder(src_tokenizer, tgt_tokenizer, train_repeat_count=100)

        model_config = {
            'num_encoder_layers': 2,
            'num_decoder_layers': 2,
            'src_vocab_size': src_tokenizer.vocab_size,
            'tgt_vocab_size': tgt_tokenizer.vocab_size,
        }

        runner = TransformerRunner(model_config, dataset_builder, model_dir='models/transformer')

        train_files = [('testdata/train.src.txt', 'testdata/train.tgt.txt')]
        runner.train(train_files, epochs=10, callbacks=None)

    def testBert(self):
        dataset_builder = BertTFRecordDatasetBuilder(
            max_sequence_length=128, record_option='GZIP', train_repeat_count=100, eos_token='T')

        model_config = {
            'max_positions': 128,
            'num_layers': 6,
            'vocab_size': 21128,
        }

        runner = BertRunner(model_config, dataset_builder, model_dir='models/bert')

        train_files = ['testdata/bert_custom_pretrain.tfrecord']
        runner.train(train_files, epochs=10, callbacks=None)

    def testAlbert(self):
        # ALBERT has the same data format with BERT
        dataset_builder = BertTFRecordDatasetBuilder(
            max_sequence_length=128, record_option='GZIP', train_repeat_count=100, eos_token='T')

        model_config = {
            'max_positions': 128,
            'num_layers': 6,
            'num_groups': 1,
            'num_layers_each_group': 1,
            'vocab_size': 21128, 
        }

        runner = AlbertRunner(model_config, dataset_builder, model_dir='models/albert')

        train_files = ['testdata/bert_custom_pretrain.tfrecord']
        runner.train(train_files, epochs=10, callbacks=None)


if __name__ == "__main__":
    tf.test.main()



# Copyright 2019 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Create masked LM/next sentence masked_lm TF examples for BERT."""
from __future__ import absolute_import, division, print_function

import collections
import random

import tensorflow as tf
from absl import app, flags, logging

from . import bert_tokenization as tokenization

FLAGS = flags.FLAGS

flags.DEFINE_string("input_file", None,
                    "Input raw text file (or comma-separated list of files).")

flags.DEFINE_string(
    "output_file", None,
    "Output TF example file (or comma-separated list of files).")

flags.DEFINE_string("vocab_file", None,
                    "The vocabulary file that the BERT model was trained on.")

flags.DEFINE_bool(
    "do_lower_case", True,
    "Whether to lower case the input text. Should be True for uncased "
    "models and False for cased models.")

flags.DEFINE_bool(
    "do_whole_word_mask", False,
    "Whether to use whole word masking rather than per-WordPiece masking.")

flags.DEFINE_bool(
    "gzip_compress", False,
    "Whether to use `GZIP` compress option to get compressed TFRecord files.")

flags.DEFINE_integer("max_seq_length", 128, "Maximum sequence length.")

flags.DEFINE_integer("max_predictions_per_seq", 20,
                     "Maximum number of masked LM predictions per sequence.")

flags.DEFINE_integer("random_seed", 12345, "Random seed for data generation.")

flags.DEFINE_integer(
    "dupe_factor", 10,
    "Number of times to duplicate the input data (with different masks).")

flags.DEFINE_float("masked_lm_prob", 0.15, "Masked LM probability.")

flags.DEFINE_float(
    "short_seq_prob", 0.1,
    "Probability of creating sequences which are shorter than the "
    "maximum length.")


class TrainingInstance(object):
    """A single training instance (sentence pair)."""

    def __init__(self, tokens, segment_ids, masked_lm_positions, masked_lm_labels,
                 is_random_next):
        self.tokens = tokens
        self.segment_ids = segment_ids
        self.is_random_next = is_random_next
        self.masked_lm_positions = masked_lm_positions
        self.masked_lm_labels = masked_lm_labels

    def __str__(self):
        s = ""
        s += "tokens: %s\n" % (" ".join(
            [tokenization.printable_text(x) for x in self.tokens]))
        s += "segment_ids: %s\n" % (" ".join([str(x) for x in self.segment_ids]))
        s += "is_random_next: %s\n" % self.is_random_next
        s += "masked_lm_positions: %s\n" % (" ".join(
            [str(x) for x in self.masked_lm_positions]))
        s += "masked_lm_labels: %s\n" % (" ".join(
            [tokenization.printable_text(x) for x in self.masked_lm_labels]))
        s += "\n"
        return s

    def __repr__(self):
        return self.__str__()


def write_instance_to_example_files(instances, tokenizer, max_seq_length,
                                    max_predictions_per_seq, output_files):
    """Create TF example files from `TrainingInstance`s."""
    writers = []
    for output_file in output_files:
        writers.append(
            tf.io.TFRecordWriter(
                output_file, options="GZIP" if FLAGS.gzip_compress else ""))

    writer_index = 0

    total_written = 0
    for (inst_index, instance) in enumerate(instances):
        input_ids = tokenizer.convert_tokens_to_ids(instance.tokens)
        input_mask = [1] * len(input_ids)
        segment_ids = list(instance.segment_ids)
        assert len(input_ids) <= max_seq_length

        while len(input_ids) < max_seq_length:
            input_ids.append(0)
            input_mask.append(0)
            segment_ids.append(0)

        assert len(input_ids) == max_seq_length
        assert len(input_mask) == max_seq_length
        assert len(segment_ids) == max_seq_length

        masked_lm_positions = list(instance.masked_lm_positions)
        masked_lm_ids = tokenizer.convert_tokens_to_ids(instance.masked_lm_labels)
        masked_lm_weights = [1.0] * len(masked_lm_ids)

        while len(masked_lm_positions) < max_predictions_per_seq:
            masked_lm_positions.append(0)
            masked_lm_ids.append(0)
            masked_lm_weights.append(0.0)

        next_sentence_label = 1 if instance.is_random_next else 0

        features = collections.OrderedDict()
        features["input_ids"] = create_int_feature(input_ids)
        features["input_mask"] = create_int_feature(input_mask)
        features["segment_ids"] = create_int_feature(segment_ids)
        features["masked_lm_positions"] = create_int_feature(masked_lm_positions)
        features["masked_lm_ids"] = create_int_feature(masked_lm_ids)
        features["masked_lm_weights"] = create_float_feature(masked_lm_weights)
        features["next_sentence_labels"] = create_int_feature([next_sentence_label])

        tf_example = tf.train.Example(features=tf.train.Features(feature=features))

        writers[writer_index].write(tf_example.SerializeToString())
        writer_index = (writer_index + 1) % len(writers)

        total_written += 1

        if inst_index < 20:
            logging.info("*** Example ***")
            logging.info("tokens: %s", " ".join(
                [tokenization.printable_text(x) for x in instance.tokens]))

            for feature_name in features.keys():
                feature = features[feature_name]
                values = []
                if feature.int64_list.value:
                    values = feature.int64_list.value
                elif feature.float_list.value:
                    values = feature.float_list.value
                logging.info("%s: %s", feature_name, " ".join([str(x) for x in values]))

    for writer in writers:
        writer.close()

    logging.info("Wrote %d total instances", total_written)


def create_int_feature(values):
    feature = tf.train.Feature(int64_list=tf.train.Int64List(value=list(values)))
    return feature


def create_float_feature(values):
    feature = tf.train.Feature(float_list=tf.train.FloatList(value=list(values)))
    return feature


def create_training_instances(input_files, tokenizer, max_seq_length,
                              dupe_factor, short_seq_prob, masked_lm_prob,
                              max_predictions_per_seq, rng):
    """Create `TrainingInstance`s from raw text."""
    all_documents = [[]]

    # Input file format:
    # (1) One sentence per line. These should ideally be actual sentences, not
    # entire paragraphs or arbitrary spans of text. (Because we use the
    # sentence boundaries for the "next sentence prediction" task).
    # (2) Blank lines between documents. Document boundaries are needed so
    # that the "next sentence prediction" task doesn't span between documents.
    for input_file in input_files:
        with tf.io.gfile.GFile(input_file, "rb") as reader:
            while True:
                line = tokenization.convert_to_unicode(reader.readline())
                if not line:
                    break
                line = line.strip()

                # Empty lines are used as document delimiters
                if not line:
                    all_documents.append([])
                tokens = tokenizer.tokenize(line)
                if tokens:
                    all_documents[-1].append(tokens)

    # Remove empty documents
    all_documents = [x for x in all_documents if x]
    rng.shuffle(all_documents)

    vocab_words = list(tokenizer.vocab.keys())
    instances = []
    for _ in range(dupe_factor):
        for document_index in range(len(all_documents)):
            instances.extend(
                create_instances_from_document(
                    all_documents, document_index, max_seq_length, short_seq_prob,
                    masked_lm_prob, max_predictions_per_seq, vocab_words, rng))

    rng.shuffle(instances)
    return instances


def create_instances_from_document(
        all_documents, document_index, max_seq_length, short_seq_prob,
        masked_lm_prob, max_predictions_per_seq, vocab_words, rng):
    """Creates `TrainingInstance`s for a single document."""
    document = all_documents[document_index]

    # Account for [CLS], [SEP], [SEP]
    max_num_tokens = max_seq_length - 3

    # We *usually* want to fill up the entire sequence since we are padding
    # to `max_seq_length` anyways, so short sequences are generally wasted
    # computation. However, we *sometimes*
    # (i.e., short_seq_prob == 0.1 == 10% of the time) want to use shorter
    # sequences to minimize the mismatch between pre-training and fine-tuning.
    # The `target_seq_length` is just a rough target however, whereas
    # `max_seq_length` is a hard limit.
    target_seq_length = max_num_tokens
    if rng.random() < short_seq_prob:
        target_seq_length = rng.randint(2, max_num_tokens)

    # We DON'T just concatenate all of the tokens from a document into a long
    # sequence and choose an arbitrary split point because this would make the
    # next sentence prediction task too easy. Instead, we split the input into
    # segments "A" and "B" based on the actual "sentences" provided by the user
    # input.
    instances = []
    current_chunk = []
    current_length = 0
    i = 0
    while i < len(document):
        segment = document[i]
        current_chunk.append(segment)
        current_length += len(segment)
        if i == len(document) - 1 or current_length >= target_seq_length:
            if current_chunk:
                # `a_end` is how many segments from `current_chunk` go into the `A`
                # (first) sentence.
                a_end = 1
                if len(current_chunk) >= 2:
                    a_end = rng.randint(1, len(current_chunk) - 1)

                tokens_a = []
                for j in range(a_end):
                    tokens_a.extend(current_chunk[j])

                tokens_b = []
                # Random next
                is_random_next = False
                if len(current_chunk) == 1 or rng.random() < 0.5:
                    is_random_next = True
                    target_b_length = target_seq_length - len(tokens_a)

                    # This should rarely go for more than one iteration for large
                    # corpora. However, just to be careful, we try to make sure that
                    # the random document is not the same as the document
                    # we're processing.
                    for _ in range(10):
                        random_document_index = rng.randint(0, len(all_documents) - 1)
                        if random_document_index != document_index:
                            break

                    random_document = all_documents[random_document_index]
                    random_start = rng.randint(0, len(random_document) - 1)
                    for j in range(random_start, len(random_document)):
                        tokens_b.extend(random_document[j])
                        if len(tokens_b) >= target_b_length:
                            break
                    # We didn't actually use these segments so we "put them back" so
                    # they don't go to waste.
                    num_unused_segments = len(current_chunk) - a_end
                    i -= num_unused_segments
                # Actual next
                else:
                    is_random_next = False
                    for j in range(a_end, len(current_chunk)):
                        tokens_b.extend(current_chunk[j])
                truncate_seq_pair(tokens_a, tokens_b, max_num_tokens, rng)

                assert len(tokens_a) >= 1
                assert len(tokens_b) >= 1

                tokens = []
                segment_ids = []
                tokens.append("[CLS]")
                segment_ids.append(0)
                for token in tokens_a:
                    tokens.append(token)
                    segment_ids.append(0)

                tokens.append("[SEP]")
                segment_ids.append(0)

                for token in tokens_b:
                    tokens.append(token)
                    segment_ids.append(1)
                tokens.append("[SEP]")
                segment_ids.append(1)

                (tokens, masked_lm_positions,
                 masked_lm_labels) = create_masked_lm_predictions(
                     tokens, masked_lm_prob, max_predictions_per_seq, vocab_words, rng)
                instance = TrainingInstance(
                    tokens=tokens,
                    segment_ids=segment_ids,
                    is_random_next=is_random_next,
                    masked_lm_positions=masked_lm_positions,
                    masked_lm_labels=masked_lm_labels)
                instances.append(instance)
            current_chunk = []
            current_length = 0
        i += 1

    return instances


MaskedLmInstance = collections.namedtuple("MaskedLmInstance",
                                          ["index", "label"])


def create_masked_lm_predictions(tokens, masked_lm_prob,
                                 max_predictions_per_seq, vocab_words, rng):
    """Creates the predictions for the masked LM objective."""

    cand_indexes = []
    for (i, token) in enumerate(tokens):
        if token == "[CLS]" or token == "[SEP]":
            continue
        # Whole Word Masking means that if we mask all of the wordpieces
        # corresponding to an original word. When a word has been split into
        # WordPieces, the first token does not have any marker and any subsequence
        # tokens are prefixed with ##. So whenever we see the ## token, we
        # append it to the previous set of word indexes.
        #
        # Note that Whole Word Masking does *not* change the training code
        # at all -- we still predict each WordPiece independently, softmaxed
        # over the entire vocabulary.
        if (FLAGS.do_whole_word_mask and len(cand_indexes) >= 1 and token.startswith("##")):
            cand_indexes[-1].append(i)
        else:
            cand_indexes.append([i])

    rng.shuffle(cand_indexes)

    output_tokens = list(tokens)

    num_to_predict = min(max_predictions_per_seq,
                         max(1, int(round(len(tokens) * masked_lm_prob))))

    masked_lms = []
    covered_indexes = set()
    for index_set in cand_indexes:
        if len(masked_lms) >= num_to_predict:
            break
        # If adding a whole-word mask would exceed the maximum number of
        # predictions, then just skip this candidate.
        if len(masked_lms) + len(index_set) > num_to_predict:
            continue
        is_any_index_covered = False
        for index in index_set:
            if index in covered_indexes:
                is_any_index_covered = True
                break
        if is_any_index_covered:
            continue
        for index in index_set:
            covered_indexes.add(index)

            masked_token = None
            # 80% of the time, replace with [MASK]
            if rng.random() < 0.8:
                masked_token = "[MASK]"
            else:
                # 10% of the time, keep original
                if rng.random() < 0.5:
                    masked_token = tokens[index]
                # 10% of the time, replace with random word
                else:
                    masked_token = vocab_words[rng.randint(0, len(vocab_words) - 1)]

            output_tokens[index] = masked_token

            masked_lms.append(MaskedLmInstance(index=index, label=tokens[index]))
    assert len(masked_lms) <= num_to_predict
    masked_lms = sorted(masked_lms, key=lambda x: x.index)

    masked_lm_positions = []
    masked_lm_labels = []
    for p in masked_lms:
        masked_lm_positions.append(p.index)
        masked_lm_labels.append(p.label)

    return (output_tokens, masked_lm_positions, masked_lm_labels)


def truncate_seq_pair(tokens_a, tokens_b, max_num_tokens, rng):
    """Truncates a pair of sequences to a maximum sequence length."""
    while True:
        total_length = len(tokens_a) + len(tokens_b)
        if total_length <= max_num_tokens:
            break

        trunc_tokens = tokens_a if len(tokens_a) > len(tokens_b) else tokens_b
        assert len(trunc_tokens) >= 1

        # We want to sometimes truncate from the front and sometimes from the
        # back to add more randomness and avoid biases.
        if rng.random() < 0.5:
            del trunc_tokens[0]
        else:
            trunc_tokens.pop()


def main(_):
    tokenizer = tokenization.FullTokenizer(
        vocab_file=FLAGS.vocab_file, do_lower_case=FLAGS.do_lower_case)

    input_files = []
    for input_pattern in FLAGS.input_file.split(","):
        input_files.extend(tf.io.gfile.glob(input_pattern))

    logging.info("*** Reading from input files ***")
    for input_file in input_files:
        logging.info("  %s", input_file)

    rng = random.Random(FLAGS.random_seed)
    instances = create_training_instances(
        input_files, tokenizer, FLAGS.max_seq_length, FLAGS.dupe_factor,
        FLAGS.short_seq_prob, FLAGS.masked_lm_prob, FLAGS.max_predictions_per_seq,
        rng)

    output_files = FLAGS.output_file.split(",")
    logging.info("*** Writing to output files ***")
    for output_file in output_files:
        logging.info("  %s", output_file)

    write_instance_to_example_files(instances, tokenizer, FLAGS.max_seq_length,
                                    FLAGS.max_predictions_per_seq, output_files)


if __name__ == "__main__":
    flags.mark_flag_as_required("input_file")
    flags.mark_flag_as_required("output_file")
    flags.mark_flag_as_required("vocab_file")
    app.run(main)



import tensorflow as tf

from .bert_dataset_builder import BertTFRecordDatasetBuilder


class BertDatasetBuilderTest(tf.test.TestCase):

    def testBertTFRecordDatasetBuilder(self):
        dataset_builder = BertTFRecordDatasetBuilder(
            record_option='GZIP',
            max_sequence_length=128,
            train_repeat_count=100,
        )

        files = ['testdata/bert_custom_pretrain.tfrecord']
        train_dataset = dataset_builder.build_train_dataset(files)
        for d in train_dataset.take(2):
            print(d)


if __name__ == "__main__":
    tf.test.main()



import tensorflow as tf

from .metrics import (MaskedSparseCategoricalAccuracy,
                      masked_sparse_categorical_accuracy)


class MetricsTest(tf.test.TestCase):

    def testMaskedSparseCategoricalAccuracy(self):
        x = tf.constant([1, 2, 3, 4, 0, 0], shape=(2, 3), dtype=tf.int32)
        y0 = tf.constant([
            [
                [.1, .5, .2, .1, .1],  # 1
                [.1, .2, .5, .1, .1],  # 2
                [.1, .2, .1, .5, .1],  # 3
            ],
            [
                [.1, .1, .1, .2, .5],  # 4
                [.5, .1, .1, .2, .1],  # 0, masked
                [.5, .1, .1, .2, .1],  # 0, masked
            ]],
            shape=(2, 3, 5),
            dtype=tf.float32
        )
        y1 = tf.constant([
            [
                [.1, .5, .2, .1, .1],  # 1
                [.1, .2, .5, .1, .1],  # 2
                [.1, .2, .1, .5, .1],  # 3
            ],
            [
                [.1, .1, .1, .2, .5],  # 4
                [.1, .1, .1, .2, .5],  # 4, masked
                [.1, .1, .5, .2, .1],  # 2, masked
            ]],
            shape=(2, 3, 5),
            dtype=tf.float32
        )
        # the last two time steps is masking positions, so these two accuracy should be equal
        acc0 = masked_sparse_categorical_accuracy(x, y0)
        acc1 = masked_sparse_categorical_accuracy(x, y1)
        self.assertEqual(acc0.numpy(), acc1.numpy())
        self.assertAlmostEqual(1.0, acc0, delta=6)
        self.assertAlmostEqual(1.0, acc1, delta=6)

        sca = tf.keras.metrics.SparseCategoricalAccuracy()
        acc = sca(x, y0)
        self.assertAlmostEqual(acc.numpy(), acc0.numpy(), delta=6)
        self.assertAlmostEqual(1.0, acc, delta=6)

        msca = MaskedSparseCategoricalAccuracy(mask_id=0, from_logits=False)
        _acc0 = msca(x, y0)
        self.assertEqual(4.0, msca.total)
        self.assertEqual(4.0, msca.count)
        self.assertEqual(4.0 / 4.0, _acc0)

        _acc1 = msca(x, y1)
        self.assertEqual(8.0, msca.total)
        self.assertEqual(8.0, msca.count)
        self.assertEqual(8.0 / 8.0, _acc1)

        y2 = tf.constant([
            [
                [.1, .5, .2, .1, .1],  # 1
                [.1, .2, .5, .1, .1],  # 2
                [.1, .2, .1, .5, .1],  # 3
            ],
            [
                [.1, .1, .5, .2, .1],  # 4
                [.1, .1, .1, .2, .5],  # 4, masked
                [.1, .1, .5, .2, .1],  # 2, masked
            ]],
            shape=(2, 3, 5),
            dtype=tf.float32
        )
        acc2 = masked_sparse_categorical_accuracy(x, y2)
        _acc2 = msca(x, y2)

        self.assertAlmostEqual(0.75, acc2, delta=6)
        self.assertEqual(11.0, msca.total)
        self.assertEqual(12.0, msca.count)
        self.assertAlmostEqual(11.0 / 12.0, _acc2, delta=6)

        print(' acc0: ', acc0.numpy())
        print(' acc1: ', acc1.numpy())
        print(' acc2: ', acc2.numpy())
        print('_acc0: ', _acc0.numpy())
        print('_acc1: ', _acc1.numpy())
        print('_acc2: ', _acc2.numpy())


if __name__ == "__main__":
    tf.test.main()
